@@ -4,9 +4,9 @@
 
 ;; Maintainer: Nicolas P. Rougier <Nicolas.Rougier@inria.fr>
 ;; URL: https://github.com/rougier/nano-modeline
-;; Version: 0.1
+;; Version: 0.2
 ;; Package-Requires: ((emacs "27.1"))
-;; Keywords: mode-line, header-line
+;; Keywords: convenience, mode-line, header-line
 
 ;; This file is not part of GNU Emacs.
 
@@ -25,10 +25,10 @@
 
 ;;; Commentary:
 ;; 
-;; Nano modeline is a custome modeline rendered as:
+;; Nano modeline is a minor mode that modify the modeline as:
 ;; [ status | name (primary)                               secondary ]
 ;;
-;; It can be displayed on bottom (mode-line) or top (header-line)
+;; It can be displayed at the bottom (mode-line) or at the top (header-line)
 ;; depending on nano-modeline-position custom setting.
 ;;
 ;; There are two sets of faces (for active and inactive modelines) that
@@ -43,13 +43,17 @@
 ;;
 ;; Usage example:
 ;;
-;; M-x: nano-modeline
+;; M-x: nano-modeline-mode
 ;;
 ;;; NEWS:
 ;;
+;; Vesrion 0.2
+;; - Implements modeline as minor mode
+;;
 ;; Version 0.1
 ;; - Submission to ELPA
 ;;
+
 ;;; Code:
 (defgroup nano nil
   "N Λ N O"
@@ -73,8 +77,6 @@ Modeline is composed as:
 [ status | name (primary)                        secondary ]"
   :group 'nano-modeline)
 
-(defvar nano-modeline--selected-window nil
-  "Currently selected window")
 
 (defcustom nano-modeline-position 'top
   "Default position (top or bottom)"
@@ -83,87 +85,106 @@ Modeline is composed as:
   :group 'nano-modeline)
 
 (defface nano-modeline-active
-  '((t (:inherit nano-subtle)))
+  '((t (:inherit mode-line)))
   "Modeline face for active modeline"
   :group 'nano-modeline-active)
 
 (defface nano-modeline-active-name
-  '((t (:inherit (nano-strong nano-modeline-active))))
+  '((t (:inherit (mode-line bold))))
   "Modeline face for active name element"
   :group 'nano-modeline-active)
 
 (defface nano-modeline-active-primary
-  '((t (:inherit (nano-default nano-modeline-active))))
+  '((t (:inherit mode-line)))
   "Modeline face for active primary element"
   :group 'nano-modeline-active)
 
 (defface nano-modeline-active-secondary
-  '((t (:inherit (nano-faded nano-modeline-active))))
+  '((t (:inherit mode-line)))
   "Modeline face for active secondary element"
   :group 'nano-modeline-active)
 
 (defface nano-modeline-active-status-RO
-  '((t (:inherit nano-popout-i)))
+  '((t (:inherit mode-line)))
   "Modeline face for active READ-ONLY element"
   :group 'nano-modeline-active)
 
 (defface nano-modeline-active-status-RW
-  '((t (:inherit nano-faded-i)))
+  '((t (:inherit mode-line)))
   "Modeline face for active READ-WRITE element"
   :group 'nano-modeline-active)
 
 (defface nano-modeline-active-status-**
-  '((t (:inherit nano-critical)))
+  '((t (:inherit mode-line)))
   "Modeline face for active MODIFIED element"
   :group 'nano-modeline-active)
 
 (defface nano-modeline-inactive
-  '((t (:inherit nano-subtle)))
+  '((t (:inherit mode-line-inactive)))
   "Modeline face for inactive window"
   :group 'nano-modeline-inactive)
 
 (defface nano-modeline-inactive-name
-  '((t (:inherit (nano-faded  nano-modeline-inactive))))
+  '((t (:inherit mode-line-inactive)))
   "Modeline face for inactive name element"
   :group 'nano-modeline-inactive)
 
 (defface nano-modeline-inactive-primary
-  '((t (:inherit (nano-faded nano-modeline-inactive))))
+  '((t (:inherit mode-line-inactive)))
   "Modeline face for inactive primary element"
   :group 'nano-modeline-inactive)
 
 (defface nano-modeline-inactive-secondary
-  '((t (:inherit (nano-faded nano-modeline-inactive))))
+  '((t (:inherit mode-line-inactive)))
   "Modeline face for inactive primary element"
   :group 'nano-modeline-inactive)
 
 (defface nano-modeline-inactive-status-RO
-  '((t (:inherit (nano-popout nano-modeline-inactive))))
+  '((t (:inherit mode-line-inactive)))
   "Modeline face for inactive READ-ONLY element"
   :group 'nano-modeline-inactive)
 
 (defface nano-modeline-inactive-status-RW
-  '((t (:inherit (nano-faded nano-modeline-inactive))))
+  '((t (:inherit mode-line-inactive)))
   "Modeline face for inactive READ-WRITE element"
   :group 'nano-modeline-inactive)
 
 (defface nano-modeline-inactive-status-**
-  '((t (:inherit (nano-critical-i nano-modeline-inactive))))
+  '((t (:inherit mode-line-inactive)))
   "Modeline face for inactive MODIFIED element"
   :group 'nano-modeline-inactive)
 
-(defun vc-branch ()
+(defcustom nano-modeline-user-mode nil
+  "User supplied mode to be evaluated for modeline."
+  :type '(choice (const nil) function)
+  :group 'nano-modeline)
+
+(defun nano-modeline-user-mode-p ()
+  "Should the user supplied mode be called for modeline?"
+  nano-modeline-user-mode)
+
+(defun nano-modeline-truncate (str size &optional ellipsis)
+  "If STR is longer than SIZE, truncate it and add ELLIPSIS."
+
+  (let ((ellipsis (or ellipsis "…")))
+    (if (> (length str) size)
+        (format "%s%s" (substring str 0 (- size (length ellipsis))) ellipsis)
+      str)))
+
+(defun nano-modeline-vc-branch ()
+  "Return current VC branch if any."
   (if vc-mode
       (let ((backend (vc-backend buffer-file-name)))
         (concat "#" (substring-no-properties vc-mode
                                  (+ (if (eq backend 'Hg) 2 3) 2))))  nil))
 
-(defun nano-mode-name ()
+(defun nano-modeline-mode-name ()
+  "Return current major mode name"
   (format-mode-line mode-name))
 
 
 ;; From https://amitp.blogspot.com/2011/08/emacs-custom-mode-line.html
-(defun shorten-directory (dir max-length)
+(defun nano-modeline-shorten-directory (dir max-length)
   "Show up to `max-length' characters of a directory name `dir'."
   (let ((path (reverse (split-string (abbreviate-file-name dir) "/")))
         (output ""))
@@ -176,6 +197,7 @@ Modeline is composed as:
       (setq output (concat "…/" output)))
     output))
 
+
 (defun nano-modeline-compose (status name primary secondary)
   "Compose a string with provided information"
   (let* ((char-width    (window-font-width nil 'header-line))
@@ -183,68 +205,55 @@ Modeline is composed as:
      (active        (eq window nano-modeline--selected-window))
          (space-up       +0.20)
          (space-down     -0.25)
-     (prefix (cond ((string= status "RO")
-            (propertize (if (window-dedicated-p)"•RO " " RO ")
-                    'face (if active 'nano-modeline-active-status-RO
-                                 'nano-modeline-inactive-status-RO)))
+         (prefix (cond ((string= status "RO")
+                        (propertize (if (window-dedicated-p)"•RO " " RO ")
+                                    'face (if active
+                                              'nano-modeline-active-status-RO
+                                            'nano-modeline-inactive-status-RO)))
                        ((string= status "**")
-            (propertize (if (window-dedicated-p)"•** " " ** ")
-                    'face (if active 'nano-modeline-active-status-**
-                                 'nano-modeline-inactive-status-**)))
+                        (propertize (if (window-dedicated-p)"•** " " ** ")
+                                    'face (if active
+                                              'nano-modeline-active-status-**
+                                            'nano-modeline-inactive-status-**)))
                        ((string= status "RW")
-            (propertize (if (window-dedicated-p) "•RW " " RW ")
-                    'face (if active 'nano-modeline-active-status-RW
-                                 'nano-modeline-inactive-status-RW)))
-               (t (propertize status
-                      'face (if active 'nano-modeline-active-status-**
-                                   'nano-modeline-inactive-status-**)))))
+                        (propertize (if (window-dedicated-p) "•RW " " RW ")
+                                    'face (if active 'nano-modeline-active-status-RW
+                                            'nano-modeline-inactive-status-RW)))
+                       (t (propertize status
+                                      'face (if active 'nano-modeline-active-status-**
+                                              'nano-modeline-inactive-status-**)))))
          (left (concat
                 (propertize " "  'face (if active 'nano-modeline-active
-                              'nano-modeline-inactive)
-                'display `(raise ,space-up))
+                                         'nano-modeline-inactive)
+                            'display `(raise ,space-up))
                 (propertize name 'face (if active 'nano-modeline-active-name
-                                  'nano-modeline-inactive-name))
+                                         'nano-modeline-inactive-name))
                 (propertize " "  'face (if active 'nano-modeline-active
-                              'nano-modeline-inactive)
-                                 'display `(raise ,space-down))
-        (propertize primary 'face (if active 'nano-modeline-active-primary
-                                                     'nano-modeline-inactive-primary))))
+                                         'nano-modeline-inactive)
+                            'display `(raise ,space-down))
+                (propertize primary 'face (if active 'nano-modeline-active-primary
+                                            'nano-modeline-inactive-primary))))
          (right (concat secondary " "))
          (available-width (- (window-total-width) 
-                 (length prefix) (length left) (length right)
-                 (/ (window-right-divider-width) char-width)))
+                             (length prefix) (length left) (length right)
+                             (/ (window-right-divider-width) char-width)))
      (available-width (max 1 available-width)))
     (concat prefix
-        left
-        (propertize (make-string available-width ?\ )
-            'face (if active 'nano-modeline-active
-                             'nano-modeline-inactive))
-        (propertize right 'face (if active 'nano-modeline-active-secondary
-                           'nano-modeline-inactive-secondary)))))
+            left
+            (propertize (make-string available-width ?\ )
+                        'face (if active 'nano-modeline-active
+                                'nano-modeline-inactive))
+            (propertize right 'face (if active 'nano-modeline-active-secondary
+                                      'nano-modeline-inactive-secondary)))))
+
 
 ;; ---------------------------------------------------------------------
-(defun nano-modeline-mu4e-dashboard-mode-p ()
-  (bound-and-true-p mu4e-dashboard-mode))
-
-(defun nano-modeline-mu4e-dashboard-mode ()
-  (nano-modeline-compose (nano-modeline-status)
-                         "Mail"
-                         (nano-modeline-mu4e-context)
-                         (format "%d messages" (plist-get mu4e~server-props :doccount))))
-
-;; ---------------------------------------------------------------------
-;; since the EIN library itself is constantly re-rendering the notebook, and thus
-;; re-setting the header-line-format, we cannot use the nano-modeline function to set
-;; the header format in a notebook buffer.  Fortunately, EIN exposes the
-;; ein:header-line-format variable for just this purpose.
-(with-eval-after-load 'ein
-  (defun nano-modeline-ein-notebook-mode ()
-    (let ((buffer-name (format-mode-line "%b")))
-      (nano-modeline-compose (if (ein:notebook-modified-p) "**" "RW")
-                             buffer-name
-                             ""
-                             (ein:header-line))))
-  (setq ein:header-line-format '((:eval (nano-modeline-ein-notebook-mode)))))
+(defun nano-modeline-ein-notebook-mode ()
+  (let ((buffer-name (format-mode-line "%b")))
+    (nano-modeline-compose (if (ein:notebook-modified-p) "**" "RW")
+                           buffer-name
+                           ""
+                           (ein:header-line))))
 
 ;; ---------------------------------------------------------------------
 (defun nano-modeline-elfeed-search-mode-p ()
@@ -256,11 +265,8 @@ Modeline is composed as:
                          (concat "(" (elfeed-search--header)  ")")
                          ""))
 
-;; Elfeed (regular header)
-(with-eval-after-load 'elfeed
-  (defun elfeed-setup-header ()
-    (setq header-line-format (default-value 'header-line-format)))
-  (setq elfeed-search-header-function #'elfeed-setup-header))
+(defun nano-modeline-elfeed-setup-header ()
+  (setq header-line-format (default-value 'header-line-format)))
 
 ;; ---------------------------------------------------------------------
 (defun nano-modeline-elfeed-show-mode-p ()
@@ -275,7 +281,7 @@ Modeline is composed as:
          (feed-title   (plist-get (elfeed-feed-meta feed) :title))
          (entry-author (elfeed-meta elfeed-show-entry :author)))
     (nano-modeline-compose (nano-modeline-status)
-                           (s-truncate 40 title "…")
+                           (nano-modeline-truncate title 40)
                            (concat "(" tags-str ")")
                            feed-title)))
 
@@ -286,19 +292,17 @@ Modeline is composed as:
 (defun nano-modeline-calendar-mode () "")
 
 ;; Calendar (no header, only overline)
-(with-eval-after-load 'calendar
-  (defun calendar-setup-header ()
-    (setq header-line-format "")
-    (face-remap-add-relative
-     'header-line `(:overline ,(face-foreground 'default)
-                    :height 0.5
-                    :background ,(face-background 'default))))
-  (add-hook 'calendar-initial-window-hook #'calendar-setup-header)
+(defun nano-modeline-calendar-setup-header ()
+  (setq header-line-format "")
+  (face-remap-add-relative
+   'header-line `(:overline ,(face-foreground 'default)
+                            :height 0.5
+                            :background ,(face-background 'default))))
 
-  ;; From https://emacs.stackexchange.com/questions/45650
-  (add-to-list 'display-buffer-alist
-               `(,(rx string-start "*Calendar*" string-end)
-                 (display-buffer-below-selected))))
+;; From https://emacs.stackexchange.com/questions/45650
+;; (add-to-list 'display-buffer-alist
+;;              `(,(rx string-start "*Calendar*" string-end)
+;;               (display-buffer-below-selected)))
 
 ;; ---------------------------------------------------------------------
 (defun nano-modeline-org-capture-mode-p ()
@@ -310,17 +314,11 @@ Modeline is composed as:
                          "(org)"
                          ""))
 
-(with-eval-after-load 'org-capture
-  (defun org-capture-turn-off-header-line ()
-    (setq-local header-line-format (default-value 'header-line-format))
-    ;; (fit-window-to-buffer nil nil 8)
-    ;; (face-remap-add-relative 'header-line '(:background "#ffffff"))
-    (message nil))
-  (add-hook 'org-capture-mode-hook
-            #'org-capture-turn-off-header-line))
+(defun nano-modeline-org-capture-turn-off-header-line ()
+  (setq-local header-line-format (default-value 'header-line-format))
+  (message nil))
 
 ;; ---------------------------------------------------------------------
-(setq Info-use-header-line nil)
 (defun nano-modeline-info-breadcrumbs ()
   (let ((nodes (Info-toc-nodes Info-current-file))
         (cnode Info-current-node)
@@ -381,7 +379,39 @@ Modeline is composed as:
   (nano-modeline-compose " >_ "
                          "Terminal"
                          (concat "(" shell-file-name ")")
-                         (shorten-directory default-directory 32)))
+                         (nano-modeline-shorten-directory default-directory 32)))
+
+
+;; ---------------------------------------------------------------------
+(defun nano-modeline-mu4e-last-query ()
+  "Get the most recent mu4e query or nil if there is none."
+  (if (fboundp 'mu4e-last-query)
+      (mu4e-last-query)
+    mu4e~headers-last-query))
+
+(defun nano-modeline-mu4e-context ()
+  "Return the current mu4e context as a non propertized string."
+
+  (if (> (length (mu4e-context-label)) 0)
+      (concat "(" (substring-no-properties (mu4e-context-label) 1 -1) ")")
+    "(none)"))
+
+(defun nano-modeline-mu4e-server-props ()
+  "Encapsulates the call to the variable mu4e-/~server-props
+depending on the version of mu4e."
+  (if (string> mu4e-mu-version "1.6.5")
+      mu4e--server-props
+    mu4e~server-props))
+
+;; ---------------------------------------------------------------------
+(defun nano-modeline-mu4e-dashboard-mode-p ()
+  (bound-and-true-p mu4e-dashboard-mode))
+
+(defun nano-modeline-mu4e-dashboard-mode ()
+  (nano-modeline-compose (nano-modeline-status)
+                         "Mail"
+                         (nano-modeline-mu4e-context)
+                         (format "%d messages" (plist-get (nano-modeline-mu4e-server-props) :doccount))))
 
 ;; ---------------------------------------------------------------------
 (defun nano-modeline-mu4e-loading-mode-p ()
@@ -404,24 +434,23 @@ Modeline is composed as:
                          (format-time-string "%A %d %B %Y, %H:%M")))
 
 ;; ---------------------------------------------------------------------
+(defun nano-modeline-mu4e-quote (str)
+  (if (string> mu4e-mu-version "1.6.5")
+      (mu4e~quote-for-modeline str)
+    (mu4e-quote-for-modeline str)))
+
 (defun nano-modeline-mu4e-headers-mode-p ()
   (derived-mode-p 'mu4e-headers-mode))
 
 (defun nano-modeline-mu4e-headers-mode ()
   (nano-modeline-compose (nano-modeline-status)
-                         (mu4e~quote-for-modeline mu4e~headers-last-query)
+                         (nano-modeline-mu4e-quote (nano-modeline-mu4e-last-query))
                          ""
                          (format "[%s]"
-                         (mu4e~quote-for-modeline
-                          (mu4e-context-name (mu4e-context-current))))
-                         ))
-
-(with-eval-after-load 'mu4e
-  (defun mu4e~header-line-format () (nano-modeline)))
+                                 (nano-modeline-mu4e-quote
+                                  (mu4e-context-name (mu4e-context-current))))))
 
 ;; ---------------------------------------------------------------------
-(setq mu4e-modeline-max-width 72)
-
 (defun nano-modeline-mu4e-view-mode-p ()
   (derived-mode-p 'mu4e-view-mode))
 
@@ -431,7 +460,7 @@ Modeline is composed as:
          (from    (mu4e~headers-contact-str (mu4e-message-field msg :from)))
          (date     (mu4e-message-field msg :date)))
     (nano-modeline-compose (nano-modeline-status)
-                           (s-truncate 40 subject "…")
+                           (nano-modeline-truncate subject 40)
                            ""
                            from)))
 
@@ -463,21 +492,22 @@ Modeline is composed as:
   (nano-modeline-compose (nano-modeline-status)
                          "Message" "(draft)" ""))
 
-
 ;; ---------------------------------------------------------------------
-(setq org-mode-line-string nil)
+(defvar org-mode-line-string nil)
 (with-eval-after-load 'org-clock
-  (add-hook 'org-clock-out-hook
-            '(lambda () (setq org-mode-line-string nil)
-                        (force-mode-line-update))))
+  (add-hook 'org-clock-out-hook #'nano-modeline-org-clock-out))
+
+(defun nano-modeline-org-clock-out ()
+  (setq org-mode-line-string nil)
+  (force-mode-line-update))
 
 (defun nano-modeline-org-clock-mode-p ()
   org-mode-line-string)
 
 (defun nano-modeline-org-clock-mode ()
     (let ((buffer-name (format-mode-line "%b"))
-          (mode-name   (nano-mode-name))
-          (branch      (vc-branch))
+          (mode-name   (nano-modeline-mode-name))
+          (branch      (nano-modeline-vc-branch))
           (position    (format-mode-line "%l:%c")))
       (nano-modeline-compose (nano-modeline-status)
                              buffer-name 
@@ -493,8 +523,8 @@ Modeline is composed as:
 
 (defun nano-modeline-docview-mode ()
   (let ((buffer-name (format-mode-line "%b"))
-    (mode-name   (nano-mode-name))
-    (branch      (vc-branch))
+    (mode-name   (nano-modeline-mode-name))
+    (branch      (nano-modeline-vc-branch))
     (page-number (concat
               (number-to-string (doc-view-current-page)) "/"
               (or (ignore-errors
@@ -515,8 +545,8 @@ Modeline is composed as:
 
 (defun nano-modeline-pdf-view-mode ()
   (let ((buffer-name (format-mode-line "%b"))
-    (mode-name   (nano-mode-name))
-    (branch      (vc-branch))
+    (mode-name   (nano-modeline-mode-name))
+    (branch      (nano-modeline-vc-branch))
     (page-number (concat
               (number-to-string (pdf-view-current-page)) "/"
               (or (ignore-errors
@@ -537,7 +567,7 @@ Modeline is composed as:
 
 (defun nano-modeline-buffer-menu-mode ()
     (let ((buffer-name "Buffer list")
-          (mode-name   (nano-mode-name))
+          (mode-name   (nano-modeline-mode-name))
           (position    (format-mode-line "%l:%c")))
 
       (nano-modeline-compose (nano-modeline-status)
@@ -547,7 +577,6 @@ Modeline is composed as:
 ;;   'header-line `(:background ,(face-background 'nano-subtle))))
 ;;(add-hook 'Buffer-menu-mode-hook
 ;;          #'buffer-menu-mode-header-line)
-(setq Buffer-menu-use-header-line nil)
 
 ;; ---------------------------------------------------------------------
 (defun nano-modeline-completion-list-mode-p ()
@@ -555,14 +584,15 @@ Modeline is composed as:
 
 (defun nano-modeline-completion-list-mode ()
     (let ((buffer-name (format-mode-line "%b"))
-          (mode-name   (nano-mode-name))
+          (mode-name   (nano-modeline-mode-name))
           (position    (format-mode-line "%l:%c")))
 
       (nano-modeline-compose (nano-modeline-status)
                              buffer-name "" position)))
+
 ;; ---------------------------------------------------------------------
 (with-eval-after-load 'deft
-  (defun deft-print-header ()
+  (defun nano-modeline-deft-print-header ()
     (force-mode-line-update)
     (widget-insert "\n")))
 
@@ -570,7 +600,7 @@ Modeline is composed as:
   (derived-mode-p 'deft-mode))
 
 (defun nano-modeline-deft-mode ()
-  (let ((prefix " RO ")
+  (let ((prefix (nano-modeline-status))
         (primary "Notes")
         (filter  (if deft-filter-regexp
                      (deft-whole-filter-regexp) "<filter>"))
@@ -578,7 +608,6 @@ Modeline is composed as:
                      (format "%d matches" (length deft-current-files))
                    (format "%d notes" (length deft-all-files)))))
     (nano-modeline-compose prefix primary filter matches)))
-    
 
 ;; ---------------------------------------------------------------------
 (defun nano-modeline-prog-mode-p ()
@@ -589,8 +618,8 @@ Modeline is composed as:
 
 (defun nano-modeline-default-mode ()
     (let ((buffer-name (format-mode-line "%b"))
-          (mode-name   (nano-mode-name))
-          (branch      (vc-branch))
+          (mode-name   (nano-modeline-mode-name))
+          (branch      (nano-modeline-vc-branch))
           (position    (format-mode-line "%l:%c")))
       (nano-modeline-compose (nano-modeline-status)
                              buffer-name
@@ -608,14 +637,6 @@ Modeline is composed as:
         (modified    (and buffer-file-name (buffer-modified-p))))
     (cond (modified  "**") (read-only "RO") (t "RW"))))
   
-;; ---------------------------------------------------------------------
-(defun nano-modeline-mu4e-context ()
-  "Return the current mu4e context as a non propertized string."
-
-  (if (> (length (mu4e-context-label)) 0)
-      (concat "(" (substring-no-properties (mu4e-context-label) 1 -1) ")")
-    "(none)"))
-
 
 ;; ---------------------------------------------------------------------
 (defun nano-modeline-face-clear (face)
@@ -627,52 +648,74 @@ Modeline is composed as:
                       :underline  'unspecified :overline   'unspecified
                       :box        'unspecified :inherit    'unspecified))
 
-(defun nano-modeline ()
-  "Install a header line whose content is dependend on the major mode"
-  (interactive)
+
+;; ---------------------------------------------------------------------
+(defvar nano-modeline--saved-mode-line-format nil)
+(defvar nano-modeline--saved-header-line-format nil)
+(defvar nano-modeline--selected-window nil)
+
+(defun nano-modeline--update-selected-window ()
+  "Update selected window (before mode-line is active)"
+  (setq nano-modeline--selected-window (selected-window)))
+
+
+(defun nano-modeline-mode--activate ()
+  "Activate nano modeline"
+
+  ;; Save current mode-line and header-line
+  (unless nano-modeline--saved-mode-line-format
+    (setq nano-modeline--saved-mode-line-format mode-line-format)
+    (setq nano-modeline--saved-header-line-format header-line-format))
+
+  ;; since the EIN library itself is constantly re-rendering the notebook, and thus
+  ;; re-setting the header-line-format, we cannot use the nano-modeline function to set
+  ;; the header format in a notebook buffer. Fortunately, EIN exposes the
+  ;; ein:header-line-format variable for just this purpose.
+  (with-eval-after-load 'ein
+    (if (eq nano-modeline-position 'top)
+        (setq ein:header-line-format '((:eval (nano-modeline-ein-notebook-mode))))))
+
+  ;; Elfeed uses header-line, we need to tell it to use our own format
+  (with-eval-after-load 'elfeed
+    (if (eq nano-modeline-position 'top)
+        (setq elfeed-search-header-function #'nano-modeline-elfeed-setup-header)))
+  
+  (with-eval-after-load 'calendar
+    (add-hook 'calendar-initial-window-hook
+              #'nano-modeline-calendar-setup-header))
+
+  (with-eval-after-load 'org-clock
+    (add-hook 'org-clock-out-hook #'nano-modeline-org-clock-out))
+  
+  (with-eval-after-load 'org-capture
+    (add-hook 'org-capture-mode-hook
+              #'nano-modeline-org-capture-turn-off-header-line))
+
+  (with-eval-after-load 'esh-mode
+    (setq eshell-status-in-mode-line nil))
+
+  (with-eval-after-load 'mu4e
+    (advice-add 'mu4e~header-line-format :override #'nano-modeline))
+
+  (if (eq nano-modeline-position 'top)
+      (setq Info-use-header-line nil))
+
+  (if (eq nano-modeline-position 'top)
+      (setq Buffer-menu-use-header-line nil))
 
   ;; Update selected window
-  (setq nano-modeline--selected-window (selected-window))
-  
-  ;; "Box" effect is obtained through display property
-  (nano-modeline-face-clear 'mode-line)
-  (nano-modeline-face-clear 'mode-line-inactive)
-  (nano-modeline-face-clear 'header-line)
-  (setq eshell-status-in-modeline nil)
-  
-        ;; TTY mode top
-  (cond ((and (not (display-graphic-p))
-              (eq nano-modeline-position 'top))
-         (setq mode-line-format nil)
-         (setq-default mode-line-format nil)
-         (set-face-attribute 'mode-line nil :inherit 'nano-modeline-active)
-         (set-face-attribute 'mode-line-inactive nil :inherit 'nano-modeline-inactive))
+  (nano-modeline--update-selected-window)
+  ;; (setq nano-modeline--selected-window (selected-window))
 
-        ;; TTY Mode bottom
-        ((and (not (display-graphic-p))
-              (eq nano-modeline-position 'top))
-         (setq header-line-format nil)
-         (setq-default header-line-format nil))
-    
-        ;; Graphic mode, modeline at top
-        ((eq nano-modeline-position 'top)
-         (setq mode-line-format (list ""))
-         (setq-default mode-line-format (list ""))
-         (set-face-attribute 'mode-line nil :inherit 'nano-modeline-active
-                                            :height 0.1)
-         (set-face-attribute 'mode-line-inactive nil :inherit 'nano-modeline-inactive
-                                                     :height 0.1))
-     
-        ;; Graphic mode, modeline at bottom
-        (t
-         (setq header-line-format nil)
-         (setq-default header-line-format nil)
-         (set-face-attribute 'mode-line nil          :height (face-attribute 'default :height))
-         (set-face-attribute 'mode-line-inactive nil :height (face-attribute 'default :height))))
-     
+  (setq         mode-line-format nil)
+  (setq-default mode-line-format nil)
+  (setq         header-line-format nil)
+  (setq-default header-line-format nil)
+      
   (let* ((format
           '((:eval
              (cond
+              ((nano-modeline-user-mode-p)            (funcall ,nano-modeline-user-mode)) 
               ((nano-modeline-prog-mode-p)            (nano-modeline-default-mode))
               ((nano-modeline-message-mode-p)         (nano-modeline-message-mode))
               ((nano-modeline-elfeed-search-mode-p)   (nano-modeline-elfeed-search-mode))
@@ -699,15 +742,60 @@ Modeline is composed as:
               (t                                      (nano-modeline-default-mode)))))))
     
     (if (eq nano-modeline-position 'top)
-        (setq-default header-line-format format)
-      (setq-default mode-line-format format))))
+        (progn 
+          (setq header-line-format format)
+          (setq-default header-line-format format))
+      (progn
+        (setq mode-line-format format)
+        (setq-default mode-line-format format)))
+
+    ;; This hooks is necessary to register selected window because when
+    ;;  a modeline is evaluated, the corresponding window is always selected.
+    (add-hook 'post-command-hook #'nano-modeline--update-selected-window)
+
+    (force-mode-line-update t)
+  ))
 
 
-;; This hooks is necessary to register selected window because when
-;;  a modeline is evaluated, the corresponding window is always selected.
-(add-hook 'post-command-hook
-      (lambda () (setq nano-modeline--selected-window (selected-window))))
+(defun nano-modeline-mode--inactivate ()
+  "Inactivate nano mode line and restored default mode-line"
+  
+  (custom-reevaluate-setting 'Info-use-header-line)
+  (custom-reevaluate-setting 'Buffer-menu-use-header-line)
+  (custom-reevaluate-setting 'eshell-status-in-mode-line)
+    
+  (setq ein:header-line-format '(:eval (ein:header-line)))
+  (setq elfeed-search-header-function #'elfeed-search--header)
+  (remove-hook 'calendar-initial-window-hook
+               #'nano-modeline-calendar-setup-header)
+  (remove-hook 'org-capture-mode-hook
+               #'nano-modeline-org-capture-turn-off-header-line)
+  (remove-hook 'org-clock-out-hook
+               #'nano-modeline-org-clock-out)
+  (remove-hook 'post-command-hook
+               #'nano-modeline--update-selected-window)
+  (advice-remove 'mu4e~header-line-format #'nano-modeline)
+
+  (setq         mode-line-format nano-modeline--saved-mode-line-format)
+  (setq-default mode-line-format nano-modeline--saved-mode-line-format)
+  (setq         header-line-format nano-modeline--saved-header-line-format)
+  (setq-default header-line-format nano-modeline--saved-header-line-format))
+
+
+;;;###autoload
+(define-minor-mode nano-modeline-mode
+  "Toggle nano-modeline minor mode"
+  :group 'nano
+  :global t
+  :init-value nil
+
+  (if nano-modeline-mode
+      (nano-modeline-mode--activate)
+    (nano-modeline-mode--inactivate))
+
+  ;; Run any registered hooks
+  (run-hooks 'nano-modeline-mode-hook))
 
-(nano-modeline)
 
 (provide 'nano-modeline)
+;;; nano-modeline.el ends here
